class Inspector(object):
    """Inspect the current view or object."""

    def show_context(self):
        yield {'key': 'context',
               'value': self.context}
        yield {'key': 'class of context',
               'value': self.context.__class__}
        yield {'key': 'base classes of context',
               'value': self.context.__class__.__bases__}



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2014 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.generations.utils
import zope.catalog
import zope.component


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(ab):
    """Update indexes due to a bug in update search result handler.

    """
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    catalog.updateIndexes()



def test_personlist__PersonTable__1(search_data, webdriver):
    """The `checkall` checkbox deselects and reselects all persons."""
    s = webdriver.login('visitor')
    s.open('/ab/@@multi_keyword.html')
    s.addSelection('id=form-widgets-keywords', 'label=family')
    s.clickAndWait('id=form-buttons-search')
    # There are two lines of search results:
    s.assertCssCount('name=persons:list', 2)
    # Checked by default:
    s.assertChecked('name=persons:list')
    # Deselect all with a single click:
    s.click('css=input.checkall')
    s.assertNotChecked('name=persons:list')
    # Select again:
    s.click('css=input.checkall')
    s.assertChecked('name=persons:list')


def test_personlist__ExportForm__1(address_book, browser):
    """The submit button of `ExportForm`' is not shown before searching."""
    browser.login('visitor')
    browser.open(browser.SEARCH_BY_KEYWORD_URL)
    assert [['form.buttons.search']] == browser.submit_control_names_all_forms


def test_personlist__ExportForm__2(address_book, KeywordFactory, browser):
    """The submit button of `ExportForm`' is not shown if nothing found."""
    KeywordFactory(address_book, u'example')
    browser.login('visitor')
    browser.keyword_search('example')
    assert [['form.buttons.search']] == browser.submit_control_names_all_forms


def test_personlist__ExportForm__3(search_data, browser):
    """The submit button of `ExportForm`' is shown if something found."""
    browser.login('visitor')
    browser.keyword_search('church')
    assert ([['form.buttons.search'], ['form.buttons.apply']] ==
            browser.submit_control_names_all_forms)


def test_personlist__ExportForm__4(search_data, browser):
    """It does not break if no handler is selected."""
    browser.login('visitor')
    browser.keyword_search('church')
    browser.getControl('Apply on selected persons').displayValue = []
    browser.getControl(name='form.buttons.apply').click()
    assert '200 Ok' == browser.headers['status']



# Copyright (c) 2008-2014 Michael Howitz
# See also LICENSE.txt
# $Id$
"""Database initialisation and upgrading."""

import zope.generations.generations


GENERATION = 28


manager = zope.generations.generations.SchemaManager(
    minimum_generation=GENERATION,
    generation=GENERATION,
    package_name='icemac.addressbook.generations')



# -*- coding: latin-1 -*-
import icemac.addressbook.generations.utils
import icemac.addressbook.utils
import zope.annotation.interfaces
import zope.authentication.interfaces
import zope.container.interfaces
import zope.dublincore.interfaces


generation = 8
MAX_DEPTH = 10


@icemac.addressbook.utils.set_site
def principal_id_to_title(id, context):
    """Convert a principal id into its title."""
    auth = zope.component.getUtility(
        zope.authentication.interfaces.IAuthentication)
    try:
        principal = auth.getPrincipal(id)
    except zope.authentication.interfaces.PrincipalLookupError:
        return id
    else:
        return principal.title


def update_object(obj, context):
    """Update one object."""
    creators = zope.dublincore.interfaces.IZopeDublinCore(obj).creators
    if not len(creators):
        return
    editor_storage = icemac.addressbook.metadata.interfaces.IEditor(obj)
    editor_storage.creator = principal_id_to_title(context, creators[0], obj)
    editor_storage.modifier = principal_id_to_title(context, creators[-1], obj)


def update_recursively(object, context, depth):
    """Do the metadata update for an object and its children."""
    depth += 1
    if depth > MAX_DEPTH:
        return
    # update object
    if zope.annotation.interfaces.IAttributeAnnotatable.providedBy(object):
        update_object(object, context)
    # update children
    if zope.container.interfaces.IContainer.providedBy(object):
        for obj in object.values():
            update_recursively(obj, context, depth)
    # update children on attributes
    if not hasattr(object, '__dict__'):
        return
    for name, obj in object.__dict__.items():
        if (zope.container.interfaces.IContainer.providedBy(object) and
                not name.startswith('_')):
            # short cut, only the values on public attributes which
            # are containers need to be converted
            update_recursively(obj, context, depth)


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Update additional metadata from existing dublincore metadata."""
    update_recursively(addressbook, addressbook, 0)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2014 Michael Howitz
# See also LICENSE.txt
# $Id$

import z3c.menu.ready2go
import zope.interface
import zope.viewlet.interfaces


class ISearchMenu(z3c.menu.ready2go.ISiteMenu):
    """Search menu."""


class ISearchForm(zope.viewlet.interfaces.IViewletManager):
    """Search form manager."""


class ISearchResult(zope.viewlet.interfaces.IViewletManager):
    """Search form manager."""


class ISearch(zope.interface.Interface):
    """A search."""

    def search(**kw):
        "Search for given keyword arguments and return iterable of results."


class ISearchMenuItemOn(zope.interface.Interface):
    """List of view names for which the search menu item in the main navigation
       should be highlighted."""



from mock import patch


def test_startpage__Dispatch__1(address_book, browser):
    """Dispatch() by default redirects to the welcome page."""
    browser.login('visitor')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert browser.ADDRESS_BOOK_WELCOME_URL == browser.url


def test_startpage__Dispatch__2(address_book, webdriver):
    """Dispatch() redirects to the page selected on the address book."""
    sel = webdriver.login('mgr')
    sel.open('/ab/@@edit-address_book.html')
    sel.select('id=form-widgets-startpage', 'label=Search')
    sel.type('id=form-widgets-title', 'Test')
    sel.clickAndWait('id=form-buttons-apply')
    sel.open('/ab')
    assert sel.getLocation().endswith('/ab/@@search.html')


def test_startpage__Dispatch__3(address_book, webdriver):
    """If accessing the selected page is not allowed Dispatch() redirects.

    It redirects to the welcome page.

    """
    sel = webdriver.login('mgr')
    sel.open('/ab/@@edit-address_book.html')
    sel.select('id=form-widgets-startpage', 'label=Search')
    sel.type('id=form-widgets-title', 'Test')
    sel.clickAndWait('id=form-buttons-apply')
    can_access_uri_part = (
        'icemac.addressbook.browser.addressbook.startpage.can_access_uri_part')
    with patch(can_access_uri_part, return_value=False):
        sel.open('/ab')
        assert sel.getLocation().endswith('/ab/@@welcome.html')



def test_resources__1(address_book, browser):
    """Image resources can be delivered to the browser."""
    browser.open('http://localhost/++resource++img/Symbol-Information.png')
    assert '200 Ok' == browser.headers['status']


def test_resources__form_js__render_field_hint__1(
        address_book, FullPersonFactory, webdriver):
    """It renders the titles of the fields as separate hint."""
    FullPersonFactory(address_book, u'Tester')
    sel = webdriver.login('editor')
    sel.open('/ab/Person')
    sel.verifyText(
        "css=#IcemacAddressbookAddressPostaladdress_0-widgets-"
        "IcemacAddressbookAddressPostaladdress_0-address_prefix-row div.hint",
        "e. g. company name or c/o")
